@@ -23,19 +23,35 @@
 #define __RB_UTIL_H
 
 #include <stdarg.h>
-#include <glib/gtypes.h>
-#include <glib-object.h>
+#include <glib.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkuimanager.h>
 
 G_BEGIN_DECLS
 
 gboolean rb_true_function (gpointer dummy);
 gboolean rb_false_function (gpointer dummy);
+gpointer rb_null_function (gpointer dummy);
 
 gboolean rb_gvalue_compare (GValue *a, GValue *b);
 
+int rb_compare_gtimeval (GTimeVal *a, GTimeVal *b);
+char * rb_make_duration_string (guint duration);
+
+void rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path);
+
 GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size);
 
+gchar *rb_uri_get_mount_point (const char *uri);
+gboolean rb_uri_is_mounted (const char *uri);
+
+
+void rb_threads_init (void);
+gboolean rb_is_main_thread (void);
+
+gchar* rb_search_fold (const char *original);
+gchar** rb_string_split_words (const gchar *string);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
