@@ -51,12 +51,11 @@ static void rb_feed_podcast_properties_dialog_update (RBFeedPodcastPropertiesDia
 static void rb_feed_podcast_properties_dialog_update_author (RBFeedPodcastPropertiesDialog *dialog);
 static void rb_feed_podcast_properties_dialog_update_language (RBFeedPodcastPropertiesDialog *dialog);
 static void rb_feed_podcast_properties_dialog_update_last_update (RBFeedPodcastPropertiesDialog *dialog);
+static void rb_feed_podcast_properties_dialog_update_last_episode (RBFeedPodcastPropertiesDialog *dialog);
 static void rb_feed_podcast_properties_dialog_update_copyright (RBFeedPodcastPropertiesDialog *dialog);
 static void rb_feed_podcast_properties_dialog_update_summary (RBFeedPodcastPropertiesDialog *dialog);
 static gchar* rb_feed_podcast_properties_dialog_parse_time (gulong time);
 
-static GtkWidget * boldify_label (GtkWidget *label);
-
 struct RBFeedPodcastPropertiesDialogPrivate
 {
 	RhythmDB *db;
@@ -67,12 +66,15 @@ struct RBFeedPodcastPropertiesDialogPrivate
 	GtkWidget   *location;
 	GtkWidget   *language;
 	GtkWidget   *last_update;
+	GtkWidget   *last_episode;
 	GtkWidget   *copyright;
 	GtkWidget   *summary;
 	
-	GtkWidget   *okbutton;
+	GtkWidget   *close_button;
 };
 
+#define RB_FEED_PODCAST_PROPERTIES_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_FEED_PODCAST_PROPERTIES_DIALOG, RBFeedPodcastPropertiesDialogPrivate))
+
 enum 
 {
 	PROP_0,
@@ -88,6 +90,8 @@ rb_feed_podcast_properties_dialog_class_init (RBFeedPodcastPropertiesDialogClass
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = rb_feed_podcast_properties_dialog_finalize;
+
+	g_type_class_add_private (klass, sizeof (RBFeedPodcastPropertiesDialogPrivate));
 }
 
 static void
@@ -95,20 +99,19 @@ rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 {
 	GladeXML *xml;
 	
-	dialog->priv = g_new0 (RBFeedPodcastPropertiesDialogPrivate, 1);
+	dialog->priv = RB_FEED_PODCAST_PROPERTIES_DIALOG_GET_PRIVATE (dialog);
 	
 	g_signal_connect_object (G_OBJECT (dialog),
 				 "response",
 				 G_CALLBACK (rb_feed_podcast_properties_dialog_response_cb),
 				 dialog, 0);
 
+	gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
+
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-					 GTK_RESPONSE_OK);
-
 	xml = rb_glade_xml_new ("podcast-feed-properties.glade",
 				"podcastproperties",
 				dialog);
@@ -117,10 +120,11 @@ rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
 			   glade_xml_get_widget (xml, "podcastproperties"));
 	
-	dialog->priv->okbutton = gtk_dialog_add_button (GTK_DIALOG (dialog),
-							GTK_STOCK_OK,
-							GTK_RESPONSE_OK);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	dialog->priv->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+							    GTK_STOCK_CLOSE,
+							    GTK_RESPONSE_CLOSE);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CLOSE);
 
 	/* get the widgets from the XML */
 	dialog->priv->title = glade_xml_get_widget (xml, "titleLabel");
@@ -128,16 +132,18 @@ rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 	dialog->priv->location = glade_xml_get_widget (xml, "locationLabel");
 	dialog->priv->language = glade_xml_get_widget (xml, "languageLabel");
 	dialog->priv->last_update = glade_xml_get_widget (xml, "lastupdateLabel");
+	dialog->priv->last_episode = glade_xml_get_widget (xml, "lastepisodeLabel");
 	dialog->priv->copyright = glade_xml_get_widget (xml, "copyrightLabel");
 	dialog->priv->summary = glade_xml_get_widget (xml, "summaryLabel");
 
-	boldify_label (glade_xml_get_widget (xml, "titleDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "authorDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "locationDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "languageDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "lastupdateDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "copyrightDescLabel"));
-	boldify_label (glade_xml_get_widget (xml, "summaryDescLabel"));
+	rb_glade_boldify_label (xml, "titleDescLabel");
+	rb_glade_boldify_label (xml, "authorDescLabel");
+	rb_glade_boldify_label (xml, "locationDescLabel");
+	rb_glade_boldify_label (xml, "languageDescLabel");
+	rb_glade_boldify_label (xml, "lastupdateDescLabel");
+	rb_glade_boldify_label (xml, "lastepisodeDescLabel");
+	rb_glade_boldify_label (xml, "copyrightDescLabel");
+	rb_glade_boldify_label (xml, "summaryDescLabel");
 
 	g_object_unref (G_OBJECT (xml));
 }
@@ -154,8 +160,6 @@ rb_feed_podcast_properties_dialog_finalize (GObject *object)
 
 	g_return_if_fail (dialog->priv != NULL);
 
-	g_free (dialog->priv);
-
 	G_OBJECT_CLASS (rb_feed_podcast_properties_dialog_parent_class)->finalize (object);
 }
 
@@ -172,17 +176,6 @@ rb_feed_podcast_properties_dialog_new (RhythmDBEntry *entry)
 	return GTK_WIDGET (dialog);
 }
 
-static GtkWidget *
-boldify_label (GtkWidget *label)
-{
-	gchar *str_final;
-	str_final = g_strdup_printf ("<b>%s</b>",
-				     gtk_label_get_label (GTK_LABEL (label)));
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
-	g_free (str_final);
-	return label;
-}
-
 static void
 rb_feed_podcast_properties_dialog_response_cb (GtkDialog *gtkdialog,
 					       int response_id,
@@ -195,12 +188,14 @@ static void
 rb_feed_podcast_properties_dialog_update (RBFeedPodcastPropertiesDialog *dialog)
 {
 	g_return_if_fail (dialog->priv->current_entry != NULL);
+
 	rb_feed_podcast_properties_dialog_update_location (dialog);
 	rb_feed_podcast_properties_dialog_update_title (dialog);
 	rb_feed_podcast_properties_dialog_update_title_label (dialog);
 	rb_feed_podcast_properties_dialog_update_author (dialog);
 	rb_feed_podcast_properties_dialog_update_language (dialog);
 	rb_feed_podcast_properties_dialog_update_last_update (dialog);
+	rb_feed_podcast_properties_dialog_update_last_episode (dialog);
 	rb_feed_podcast_properties_dialog_update_copyright (dialog);
 	rb_feed_podcast_properties_dialog_update_summary (dialog);
 }
@@ -211,7 +206,7 @@ rb_feed_podcast_properties_dialog_update_title (RBFeedPodcastPropertiesDialog *d
 	const char *name;
 	char *tmp;	
 	name = rb_refstring_get (dialog->priv->current_entry->title);
-	tmp = g_strdup_printf (_("Properties for %s"), name);
+	tmp = g_strdup_printf (_("%s Properties"), name);
 	gtk_window_set_title (GTK_WINDOW (dialog), tmp);
 	g_free (tmp);
 }
@@ -256,12 +251,25 @@ rb_feed_podcast_properties_dialog_update_language (RBFeedPodcastPropertiesDialog
 static void
 rb_feed_podcast_properties_dialog_update_last_update (RBFeedPodcastPropertiesDialog *dialog)
 {
-	char *time;
+	char *time_str;
+	gulong time_val;
 
-	time = rb_feed_podcast_properties_dialog_parse_time (dialog->priv->current_entry->podcast->post_time);
-	gtk_label_set (GTK_LABEL (dialog->priv->last_update),
-		       time);
-	g_free (time);
+	time_val = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_LAST_SEEN);
+	time_str = rb_feed_podcast_properties_dialog_parse_time (time_val);
+	gtk_label_set (GTK_LABEL (dialog->priv->last_update), time_str);
+	g_free (time_str);
+}
+
+static void
+rb_feed_podcast_properties_dialog_update_last_episode (RBFeedPodcastPropertiesDialog *dialog)
+{
+	char *time_str;
+	gulong time_val;
+
+	time_val = rhythmdb_entry_get_ulong (dialog->priv->current_entry, RHYTHMDB_PROP_POST_TIME);
+	time_str = rb_feed_podcast_properties_dialog_parse_time (time_val);
+	gtk_label_set (GTK_LABEL (dialog->priv->last_episode), time_str);
+	g_free (time_str);
 }
 
 static void
@@ -275,14 +283,12 @@ static char *
 rb_feed_podcast_properties_dialog_parse_time (gulong value)
 {
 	char *str;
-	struct tm then;
 
 	if (0 == value) {
-		return NULL;
+		str = g_strdup (_("Unknown"));
+	} else {
+		str = rb_utf_friendly_time ((time_t)value);
 	}
 
-	localtime_r ((time_t*)&value, &then);
-	str = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), &then);
-
 	return str;
 }
