@@ -93,6 +93,8 @@ struct RBPodcastManagerPrivate
 	GAsyncQueue *event_queue;
 };
 
+#define RB_PODCAST_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PODCAST_MANAGER, RBPodcastManagerPrivate))
+
 /* used on event loop */
 typedef struct
 {
@@ -260,14 +262,16 @@ rb_podcast_manager_class_init (RBPodcastManagerClass *klass)
 				g_cclosure_marshal_VOID__STRING,
 				G_TYPE_NONE,
 				1,
-				G_TYPE_POINTER);
-					
+				G_TYPE_STRING);
+
+	g_type_class_add_private (klass, sizeof (RBPodcastManagerPrivate));
 }	
 
 static void
 rb_podcast_manager_init (RBPodcastManager *pd)
 {
-	pd->priv = g_new0 (RBPodcastManagerPrivate, 1);
+	pd->priv = RB_PODCAST_MANAGER_GET_PRIVATE (pd);
+
 	pd->priv->source_sync = 0;
 	pd->priv->mutex_job = g_mutex_new();
 	pd->priv->download_list_mutex = g_mutex_new();
@@ -278,7 +282,7 @@ rb_podcast_manager_init (RBPodcastManager *pd)
 
 static GObject *
 rb_podcast_manager_constructor (GType type, guint n_construct_properties,
-			   GObjectConstructParam *construct_properties)
+				GObjectConstructParam *construct_properties)
 {
 	RBPodcastManager *pd;
 
@@ -326,23 +330,20 @@ rb_podcast_manager_finalize (GObject *object)
 	g_mutex_free (pd->priv->mutex_job);	
 	g_mutex_free (pd->priv->download_list_mutex);	
 	g_async_queue_unref (pd->priv->event_queue);
-	
-	g_free (pd->priv);
-	
+
 	G_OBJECT_CLASS (rb_podcast_manager_parent_class)->finalize (object);
 	rb_debug ("Podcast Manager END");
 }
 
 static void 
 rb_podcast_manager_set_property (GObject *object,
-                           		guint prop_id,
-	                                const GValue *value,
-	                                GParamSpec *pspec)
+				 guint prop_id,
+				 const GValue *value,
+				 GParamSpec *pspec)
 {
 	RBPodcastManager *pd = RB_PODCAST_MANAGER (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		if (pd->priv->db) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (pd->priv->db),
@@ -381,8 +382,7 @@ rb_podcast_manager_get_property (GObject *object,
 {
 	RBPodcastManager *pd = RB_PODCAST_MANAGER (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, pd->priv->db);
 		break;
@@ -432,6 +432,19 @@ rb_podcast_manager_download_entry (RBPodcastManager *pd, RhythmDBEntry *entry)
 	}
 }
 
+gboolean
+rb_podcast_manager_entry_downloaded (RhythmDBEntry *entry)
+{
+	g_assert (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
+
+	gulong status;
+	const gchar *file_name;
+
+	status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
+	file_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
+
+	return (status != RHYTHMDB_PODCAST_STATUS_ERROR && file_name != NULL);
+}
 
 void
 rb_podcast_manager_start_sync (RBPodcastManager *pd)
@@ -810,6 +823,8 @@ rb_podcast_manager_add_post (RhythmDB *db,
 		entry = rhythmdb_entry_new (db,
 					    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
 				    	    uri);
+		if (entry == NULL)
+			return FALSE;
 
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, name);
@@ -1253,7 +1268,14 @@ rb_podcast_manager_db_entry_deleted_cb (RBPodcastManager *pd, RhythmDBEntry *ent
 		const gchar *conf_dir_name;
 		GnomeVFSURI *uri;
 
+		/* make sure we're not downloading it */
+		rb_podcast_manager_cancel_download (pd, entry);
+
 		file_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
+		if (file_name == NULL) {
+			/* episode has not been downloaded */
+			return;
+		}
 		
 		uri = gnome_vfs_uri_new (file_name);
 		
@@ -1384,6 +1406,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	GValue author_val = { 0, };
 	GValue status_val = { 0, };
 	GValue last_post_val = { 0, };
+	GValue last_update_val = { 0, };
 	gulong last_post = 0;
 	gulong new_last_post;
 	gboolean new_feed, updated;
@@ -1412,13 +1435,15 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 		g_value_set_ulong (&status_val, 1);
 		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_STATUS, &status_val);
 		g_value_unset (&status_val);
-		last_post = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_SEEN);
+		last_post = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
 		new_feed = FALSE;
 	} else {
 		rb_debug ("Insert new entry");
 		entry = rhythmdb_entry_new (db,
 					    RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
 				    	    (gchar*) data->url);
+		if (entry == NULL)
+			return;
 		rb_debug("New entry create\n");
 	
 		g_value_init (&title_val, G_TYPE_STRING);
@@ -1492,7 +1517,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	for (lst_songs = data->posts; lst_songs != NULL; lst_songs = g_list_next (lst_songs)) {
 		RBPodcastItem *item = (RBPodcastItem *) lst_songs->data;
 
-		if (item->pub_date > last_post) {
+		if (item->pub_date > last_post || item->pub_date == 0) {
 			gulong status;
 			updated = TRUE;
 
@@ -1529,10 +1554,19 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	g_value_set_ulong (&last_post_val, new_last_post);
 
 	if (new_feed) 
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_SEEN, &last_post_val);
+		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_POST_TIME, &last_post_val);
 	else
-		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_SEEN, &last_post_val);
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_POST_TIME, &last_post_val);
 	g_value_unset (&last_post_val);
+
+	g_value_init (&last_update_val, G_TYPE_ULONG);
+	g_value_set_ulong (&last_update_val, time(NULL));
+
+	if (new_feed)
+		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_SEEN, &last_update_val);
+	else
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_SEEN, &last_update_val);
+	g_value_unset (&last_update_val);
 	
 	rhythmdb_commit (db);
 }
