@@ -56,7 +56,7 @@ typedef struct
 	void	(*complete)		(void);
 	void	(*entry_prop_changed)	(RhythmDBEntry *entry, RhythmDBPropType prop,
 					 const GValue *old, const GValue *new);
-	void    (*non_entry_dropped)    (const char *uri);
+	void    (*non_entry_dropped)    (const char *uri, int position);
 	void    (*entry_removed)        (RhythmDBEntry *entry);
 
 } RhythmDBQueryModelClass;
@@ -65,16 +65,17 @@ GType			rhythmdb_query_model_get_type	(void);
 
 RhythmDBQueryModel *	rhythmdb_query_model_new	(RhythmDB *db, GPtrArray *query,
 							 GCompareDataFunc sort_func,
-							 gpointer user_data);
+							 RhythmDBPropType sort_prop_id,
+							 gboolean sort_reverse);
 
 RhythmDBQueryModel *	rhythmdb_query_model_new_empty	(RhythmDB *db);
 
-void			rhythmdb_query_model_add_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
+void			rhythmdb_query_model_add_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry, gint index);
 void			rhythmdb_query_model_add_entries(RhythmDBQueryModel *model, GPtrArray *entries);
 
 gboolean		rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
 
-void			rhythmdb_query_model_set_connected	(RhythmDBQueryModel *model, gboolean connected);
+void			rhythmdb_query_model_move_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry, gint index);
 
 void			rhythmdb_query_model_signal_complete	(RhythmDBQueryModel *model);
 
@@ -88,6 +89,49 @@ gboolean		rhythmdb_query_model_entry_to_iter(RhythmDBQueryModel *model,
 
 gboolean		rhythmdb_query_model_has_pending_changes (RhythmDBQueryModel *model);
 
+RhythmDBEntry *		rhythmdb_query_model_tree_path_to_entry(RhythmDBQueryModel *model,
+								GtkTreePath *path);
+RhythmDBEntry *		rhythmdb_query_model_iter_to_entry(RhythmDBQueryModel *model,
+							   GtkTreeIter *entry_iter);
+RhythmDBEntry *		rhythmdb_query_model_get_next_from_entry(RhythmDBQueryModel *model,
+								 RhythmDBEntry *entry);
+RhythmDBEntry *		rhythmdb_query_model_get_previous_from_entry(RhythmDBQueryModel *model,
+								     RhythmDBEntry *entry);
+char *			rhythmdb_query_model_compute_status_normal (RhythmDBQueryModel *model);
+
+void			rhythmdb_query_model_set_sort_order (RhythmDBQueryModel *model,
+							     GCompareDataFunc sort_func,
+							     RhythmDBPropType sort_prop_id,
+							     gboolean sort_reverse);
+
+void			rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter);
+
+gint 			rhythmdb_query_model_location_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+								 RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_string_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							       RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							      RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							       RhythmDBQueryModel *model);
+				
+gint 			rhythmdb_query_model_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							      RhythmDBQueryModel *model); 
+
+gint 			rhythmdb_query_model_track_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							      RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+								       RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_ulong_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							      RhythmDBQueryModel *model);
+
+gint 			rhythmdb_query_model_date_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+							     RhythmDBQueryModel *model);
 G_END_DECLS
 
-#endif /* __RHYTHMBDB_QUERY_MODEL_H */
+#endif /* __RHYTHMDB_QUERY_MODEL_H */
