@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of RhythmDB tree-structured database
  *
  *  Copyright (C) 2003, 2004 Colin Walters <walters@verbum.org>
@@ -33,7 +34,7 @@
 #include <string.h>
 #include <glib/gprintf.h>
 #include <glib/gatomic.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkliststore.h>
 #include <libxml/entities.h>
 #include <libxml/SAX.h>
@@ -111,8 +112,6 @@ static gboolean evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *quer
 
 struct RhythmDBTreePrivate
 {
-	GMemChunk *property_memchunk;
-
 	GHashTable *entries;
 	GHashTable *genres;
 	GMutex *genres_lock;
@@ -121,6 +120,8 @@ struct RhythmDBTreePrivate
 	guint idle_load_id;
 };
 
+#define RHYTHMDB_TREE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RHYTHMDB_TYPE_TREE, RhythmDBTreePrivate))
+
 enum
 {
 	PROP_0,
@@ -147,18 +148,17 @@ rhythmdb_tree_class_init (RhythmDBTreeClass *klass)
 	rhythmdb_class->impl_entry_foreach = rhythmdb_tree_entry_foreach;
 	rhythmdb_class->impl_evaluate_query = rhythmdb_tree_evaluate_query;
 	rhythmdb_class->impl_do_full_query = rhythmdb_tree_do_full_query;
+
+	g_type_class_add_private (klass, sizeof (RhythmDBTreePrivate));
 }
 
 static void
 rhythmdb_tree_init (RhythmDBTree *db)
 {
-	db->priv = g_new0 (RhythmDBTreePrivate, 1);
+	db->priv = RHYTHMDB_TREE_GET_PRIVATE (db);
 
 	db->priv->entries = g_hash_table_new (g_str_hash, g_str_equal);
 
-	db->priv->property_memchunk = g_mem_chunk_new ("RhythmDBTree property memchunk",
-						       sizeof (RhythmDBTreeProperty),
-						       1024, G_ALLOC_AND_FREE);
 	db->priv->genres = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 						  NULL, (GDestroyNotify)g_hash_table_destroy);
 }
@@ -187,12 +187,8 @@ rhythmdb_tree_finalize (GObject *object)
 
 	g_hash_table_destroy (db->priv->entries);
 
-	g_mem_chunk_destroy (db->priv->property_memchunk);
-
 	g_hash_table_destroy (db->priv->genres);
 
-	g_free (db->priv);
-
 	G_OBJECT_CLASS (rhythmdb_tree_parent_class)->finalize (object);
 }
 
@@ -326,9 +322,23 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			rb_debug ("pre-Date entry found, causing re-read");
 			ctx->entry->mtime = 0;
 		}
-		rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
-		rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
-		rhythmdb_commit (RHYTHMDB (ctx->db));
+		if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED && ctx->entry->podcast->post_time == 0) {
+			/* Handle upgrades from 0.9.2.
+			 * Previously, last-seen for podcast feeds was the time of the last post,
+			 * and post-time was unused.  Now, we want last-seen to be the time we
+			 * last updated the feed, and post-time to be the time of the last post.
+			 */
+			ctx->entry->podcast->post_time = ctx->entry->last_seen;
+		}
+		
+		if (ctx->entry->location != NULL) {
+			rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
+			rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
+			rhythmdb_commit (RHYTHMDB (ctx->db));
+		} else {
+			rb_debug ("found entry without location");
+			rhythmdb_entry_unref (RHYTHMDB (ctx->db), ctx->entry);
+		}
 		ctx->state = RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB;
 		break;
 	}
@@ -956,7 +966,7 @@ rhythmdb_tree_entry_new (RhythmDB *rdb, RhythmDBEntry *entry)
 static RhythmDBTreeProperty *
 rhythmdb_tree_property_new (RhythmDBTree *db)
 {
-	RhythmDBTreeProperty *ret = g_mem_chunk_alloc0 (db->priv->property_memchunk);
+	RhythmDBTreeProperty *ret = g_new0 (RhythmDBTreeProperty, 1);
 #ifndef G_DISABLE_ASSERT
 	ret->magic = 0xf00dbeef;
 #endif	
@@ -1278,6 +1288,7 @@ destroy_tree_property (RhythmDBTreeProperty *prop)
 	prop->magic = 0xf33df33d;
 #endif
 	g_hash_table_destroy (prop->children);
+	g_free (prop);
 }
 
 typedef void (*RhythmDBTreeTraversalFunc) (RhythmDBTree *db, RhythmDBEntry *entry, gpointer data);
@@ -1478,6 +1489,9 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 			break;
 		case RHYTHMDB_QUERY_END:
 		case RHYTHMDB_QUERY_DISJUNCTION:
+		case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
+		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
+		case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
 			g_assert_not_reached ();
 			break;
 		}
@@ -1749,6 +1763,9 @@ do_query_recurse (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeTraversalFunc
 
 	rb_debug ("doing recursive query, %d conjunctions", g_list_length (conjunctions));
 
+	if (conjunctions == NULL)
+		return;
+
 	/* If there is a disjunction involved, we must uniquify the entry hits. */
 	if (conjunctions->next != NULL)
 		data->entries = g_hash_table_new (g_direct_hash, g_direct_equal);
