@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: The Rhythmbox main entrypoint
  *
  *  Copyright (C) 2002 Jorn Baayen
@@ -21,25 +22,28 @@
  */
 
 #include <config.h>
-#include <libintl.h>
-#include <locale.h>
-#include <libgnome/gnome-program.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h> /* For _get_user_time... */
-#include <glade/glade-init.h>
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
+#include <libintl.h>
+#include <locale.h>
+
+#include <glib/gi18n.h>
+#include <gdk/gdkx.h> /* For _get_user_time... */
+#include <gtk/gtk.h>
+#include <glade/glade-init.h>
+#include <libgnome/gnome-program.h>
+#include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-authentication-manager.h>
+
 #ifdef HAVE_GSTREAMER
 #include <gst/gst.h>
+#ifdef HAVE_GSTREAMER_0_8
 #include <gst/gconf/gconf.h>
 #include <gst/control/control.h>
 #endif
-#ifdef WITH_MONKEYMEDIA
-#include "monkey-media.h"
 #endif
 #ifdef WITH_RHYTHMDB_GDA
 #include <libgda/libgda.h>
@@ -74,7 +78,9 @@ static gboolean no_registration = FALSE;
 static gboolean no_update	= FALSE;
 static gboolean dry_run		= FALSE;
 static char *rhythmdb_file = NULL;
+#if WITH_DBUS || WITH_BONOBO
 static gboolean load_uri_args (const char **args, GFunc handler, gpointer user_data);
+#endif
 
 #ifdef WITH_BONOBO
 static gboolean print_playing = FALSE;
@@ -167,7 +173,7 @@ main (int argc, char **argv)
 		{ "dry-run",			0,  POPT_ARG_NONE,	&dry_run,			0, N_("Don't save any data permanently (implies --no-registration)"), NULL },
 		{ "rhythmdb-file",		0,  POPT_ARG_STRING,	&rhythmdb_file,			0, N_("Path for database file to use"), NULL },
 		{ "quit",			'q',POPT_ARG_NONE,	&quit,				0, N_("Quit Rhythmbox"), NULL },
-#ifdef HAVE_GSTREAMER
+#ifdef HAVE_GSTREAMER_0_8
 		{NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL},
 #endif
 		POPT_TABLEEND
@@ -180,9 +186,14 @@ main (int argc, char **argv)
 	new_argv[argc] = g_strdup ("--disable-sound");
 	new_argv[argc+1] = NULL;
 
-#ifdef HAVE_GSTREAMER	
+#ifdef HAVE_GSTREAMER_0_8
 	popt_options[(sizeof(popt_options)/sizeof(popt_options[0]))-2].arg
 		= (void *) gst_init_get_popt_table ();
+#elif HAVE_GSTREAMER_0_10
+        /* To pass options to GStreamer in 0.10, RB needs to use GOption, for
+         * now, GStreamer can live without them (people can still use env
+         * vars, after all) */
+        gst_init (NULL, NULL);
 #endif
 #ifdef WITH_BONOBO
 	rb_remote_bonobo_preinit ();
@@ -200,13 +211,7 @@ main (int argc, char **argv)
                                g_value_init (&context_as_value, G_TYPE_POINTER));
         poptContext = g_value_get_pointer (&context_as_value);
 
-	/* Disabled because it breaks internet radio and other things -
-	 * doing synchronous calls in the main thread causes deadlock.
-	 * This is too hard to fix right now, so we punt.
-	 */
-#if 0
 	gnome_authentication_manager_init ();
-#endif
 
 	g_random_set_seed (time(0));
 
@@ -217,12 +222,9 @@ main (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
-#ifdef HAVE_GSTREAMER	
+#ifdef HAVE_GSTREAMER_0_8
 	gst_control_init (&argc, &argv);
 #endif
-#ifdef WITH_MONKEYMEDIA
-	monkey_media_init (&argc, &argv);
-#endif
 
 	rb_debug_init (debug);
 	rb_debug ("initializing Rhythmbox %s", VERSION);
@@ -250,23 +252,31 @@ main (int argc, char **argv)
 
 	session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	if (session_bus == NULL) {
-		g_critical ("couldn't connect to session bus: %s", error->message);
+		g_critical ("couldn't connect to session bus: %s", (error) ? error->message : "(null)");
 		g_clear_error (&error);
 	} else if (!no_registration) {
 		guint request_name_reply;
+		int flags;
 		DBusGProxy *bus_proxy;
 
 		bus_proxy = dbus_g_proxy_new_for_name (session_bus,
 						       "org.freedesktop.DBus",
 						       "/org/freedesktop/DBus",
 						       "org.freedesktop.DBus");
+
+#ifndef DBUS_NAME_FLAG_DO_NOT_QUEUE
+		flags = DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT;
+#else
+		flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
+#endif                
+
 		if (!dbus_g_proxy_call (bus_proxy,
 					"RequestName",
 					&error,
 					G_TYPE_STRING,
 					"org.gnome.Rhythmbox",
 					G_TYPE_UINT,
-					DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT,
+					flags,
 					G_TYPE_INVALID,
 					G_TYPE_UINT,
 					&request_name_reply,
@@ -308,19 +318,21 @@ main (int argc, char **argv)
 		rb_shell = rb_shell_new (argc, argv, TRUE, no_update, dry_run, rhythmdb_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
 #ifdef WITH_BONOBO
-		rb_remote_bonobo_acquire (bonobo, RB_REMOTE_PROXY (rb_shell), &error);
-		if (error) {
-			g_warning ("error: %s", error->message);
-		} else {
-			if (rb_remote_bonobo_activate (bonobo))
-				client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
-			else
-				g_critical ("acquired bonobo service but couldn't activate!");
+		if (!no_registration) {
+			rb_remote_bonobo_acquire (bonobo, RB_REMOTE_PROXY (rb_shell), &error);
+			if (error) {
+				g_warning ("error: %s", error->message);
+			} else {
+				if (rb_remote_bonobo_activate (bonobo))
+					client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
+				else
+					g_critical ("acquired bonobo service but couldn't activate!");
+			}
 		}
 
 #endif
 #ifdef WITH_DBUS
-		if (session_bus != NULL) {
+		if (!no_registration && session_bus != NULL) {
 			GObject *player;
 			const char *path;
 
@@ -333,7 +345,7 @@ main (int argc, char **argv)
 			dbus_g_connection_register_g_object (session_bus, path, G_OBJECT (player));
 		}
 #endif
-	} else {
+	} else if (!no_registration) {
 #ifdef WITH_DBUS
 		DBusGProxy *shell_proxy;
 		guint32 current_time;
@@ -349,6 +361,9 @@ main (int argc, char **argv)
 		} else {
 			load_uri_args (poptGetArgs (poptContext), (GFunc) dbus_load_uri, shell_proxy);
 		}
+#ifdef WITH_BONOBO
+		if (!client_proxy) {
+#endif /* WITH_BONOBO */
 #if GTK_MINOR_VERSION >= 8
 		current_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
 #else
@@ -360,14 +375,18 @@ main (int argc, char **argv)
 		 * ourself...
 		 */
 		current_time = GDK_CURRENT_TIME;
-#endif
+#endif	/* GTK_MINOR_VERSION */
 		dbus_g_proxy_call_no_reply (shell_proxy, "present",
 					    G_TYPE_UINT, current_time,
 					    G_TYPE_INVALID);
-#endif
+#ifdef WITH_BONOBO
+		}
+#endif /* WITH_BONOBO */
+#endif /* WITH_DBUS */
 	}
 #ifdef WITH_BONOBO	
-	handle_cmdline (client_proxy, activated, poptGetArgs (poptContext));
+	if (client_proxy)
+		handle_cmdline (client_proxy, activated, poptGetArgs (poptContext));
 #endif
 
 	if (activated) {
@@ -396,6 +415,7 @@ main (int argc, char **argv)
 	exit (0);
 }
 
+#if WITH_DBUS || WITH_BONOBO
 static gboolean
 load_uri_args (const char **args, GFunc handler, gpointer user_data)
 {
@@ -419,6 +439,7 @@ load_uri_args (const char **args, GFunc handler, gpointer user_data)
 	}
 	return handled;
 }
+#endif
 
 #ifdef WITH_DBUS
 static void
