@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of Song History List
  *
  *  Copyright (C) 2003 Jeffrey Yasskin <jyasskin@mail.utexas.edu>
@@ -19,10 +20,11 @@
  *  
  */ 
 
+#include <string.h>
+
 #include "rb-history.h"
 
 #include "rhythmdb.h"
-#include <string.h>
 #include "gsequence.h"
 
 struct RBHistoryPrivate
@@ -40,6 +42,8 @@ struct RBHistoryPrivate
 	gpointer destroy_userdata;
 };
 
+#define RB_HISTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_HISTORY, RBHistoryPrivate))
+
 #define MAX_HISTORY_SIZE 50
 
 static void rb_history_class_init (RBHistoryClass *klass);
@@ -69,43 +73,13 @@ enum
 	PROP_MAX_SIZE,
 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rb_history_get_type (void)
-{
-	static GType rb_history_type = 0;
-
-	if (rb_history_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBHistoryClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_history_class_init,
-			NULL,
-			NULL,
-			sizeof (RBHistory),
-			0,
-			(GInstanceInitFunc) rb_history_init
-		};
-
-		rb_history_type = g_type_register_static (G_TYPE_OBJECT,
-							  "RBHistory",
-							  &our_info, 0);
-	}
-
-	return rb_history_type;
-}
+G_DEFINE_TYPE (RBHistory, rb_history, G_TYPE_OBJECT)
 
 static void
 rb_history_class_init (RBHistoryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_history_finalize;
 
 	object_class->set_property = rb_history_set_property;
@@ -128,6 +102,7 @@ rb_history_class_init (RBHistoryClass *klass)
 							    0,
 							    G_PARAM_READWRITE));
 
+	g_type_class_add_private (klass, sizeof (RBHistoryPrivate));
 }
 
 RBHistory *
@@ -150,7 +125,7 @@ rb_history_new (gboolean truncate_on_play, GFunc destroyer, gpointer destroy_use
 static void
 rb_history_init (RBHistory *hist)
 {
-	hist->priv = g_new0 (RBHistoryPrivate, 1);
+	hist->priv = RB_HISTORY_GET_PRIVATE (hist);
 
 	hist->priv->entry_to_seqptr = g_hash_table_new (g_direct_hash,
 							g_direct_equal);
@@ -172,9 +147,8 @@ rb_history_finalize (GObject *object)
 
 	g_hash_table_destroy (hist->priv->entry_to_seqptr);
 	g_sequence_free (hist->priv->seq);
-	g_free (hist->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_history_parent_class)->finalize (object);
 }
 
 static void
@@ -373,8 +347,12 @@ void
 rb_history_set_playing (RBHistory *hist, RhythmDBEntry *entry)
 {
 	g_return_if_fail (RB_IS_HISTORY (hist));
-	g_return_if_fail (entry != NULL);
 
+	if (entry == NULL) {
+		hist->priv->current = g_sequence_get_end_ptr (hist->priv->seq);
+		return;
+	}
+	
 	rb_history_remove_entry (hist, entry);
 
 	g_sequence_insert (g_sequence_ptr_next (hist->priv->current), entry);
@@ -407,9 +385,6 @@ rb_history_append (RBHistory *hist, RhythmDBEntry *entry)
 	new_node = g_sequence_ptr_prev (g_sequence_get_end_ptr (hist->priv->seq));
 	g_hash_table_insert (hist->priv->entry_to_seqptr, entry, new_node);
 
-	if (g_sequence_ptr_is_end (hist->priv->current))
-		hist->priv->current = new_node;
-
 	rb_history_limit_size (hist, TRUE);
 }
 
@@ -486,7 +461,7 @@ static void
 rb_history_delete_link (RBHistory *hist, GSequencePtr to_delete)
 {
 	if (to_delete == hist->priv->current) {
-		hist->priv->current = g_sequence_ptr_next (hist->priv->current);
+		hist->priv->current = g_sequence_get_end_ptr (hist->priv->seq);
 	}
 	g_assert (to_delete != hist->priv->current);
 
