@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of Rhythmbox playlist management object
  *
  *  Copyright (C) 2003,2004 Colin Walters <walters@gnome.org>
@@ -21,17 +21,22 @@
  */
 
 #include <config.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <libxml/tree.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
+
 #include <string.h>
 #include <stdio.h>      /* rename() */
 #include <unistd.h>     /* unlink() */
 
+#include <libxml/tree.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
+
 #include "rb-playlist-manager.h"
 #include "rb-playlist-source.h"
+#include "rb-static-playlist-source.h"
+#include "rb-auto-playlist-source.h"
+#include "rb-play-queue-source.h"
 #include "rb-recorder.h"
 #include "rb-sourcelist.h"
 #include "rb-sourcelist-model.h"
@@ -97,7 +102,7 @@ struct RBPlaylistManagerPrivate
 
 	guint playlist_serial;
 
-	RBPlaylistSource *loading_playlist;
+	RBStaticPlaylistSource *loading_playlist;
 
 	char *firsturi;
 
@@ -109,10 +114,13 @@ struct RBPlaylistManagerPrivate
 	GCond *saving_condition;
 	GMutex *saving_mutex;
 
+	gboolean exiting;
 	gboolean saving;
 	gboolean dirty;
 };
 
+#define RB_PLAYLIST_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PLAYLIST_MANAGER, RBPlaylistManagerPrivate))
+
 enum
 {
 	PROP_0,
@@ -139,7 +147,7 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	/* Submenu of Music */
 	{ "Playlist", NULL, N_("_Playlist") },
 
-	{ "MusicPlaylistNewPlaylist", GNOME_MEDIA_PLAYLIST, N_("_New Playlist..."), "<control>N",
+	{ "MusicPlaylistNewPlaylist", GNOME_MEDIA_PLAYLIST, N_("_New Playlist"), "<control>N",
 	  N_("Create a new playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_playlist) },
 	{ "MusicPlaylistNewAutomaticPlaylist", GNOME_MEDIA_AUTO_PLAYLIST, N_("New _Automatic Playlist..."), NULL,
@@ -154,55 +162,25 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	{ "MusicPlaylistBurnPlaylist", GTK_STOCK_CDROM, N_("_Create Audio CD..."), NULL,
 	  N_("Create an audio CD from playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_burn_playlist) },
-	{ "MusicPlaylistRenamePlaylist", NULL, N_("_Rename..."), NULL,
+	{ "MusicPlaylistRenamePlaylist", NULL, N_("_Rename"), NULL,
 	  N_("Rename playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_rename_playlist) },
 	{ "MusicPlaylistDeletePlaylist", GTK_STOCK_REMOVE, N_("_Delete"), NULL,
 	  N_("Delete playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_delete_playlist) },
-	{ "EditAutomaticPlaylist", GTK_STOCK_PROPERTIES, N_("_Edit"), NULL,
+	{ "EditAutomaticPlaylist", GTK_STOCK_PROPERTIES, N_("_Edit..."), NULL,
 	  N_("Change this automatic playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_edit_automatic_playlist) },
 };
 static guint rb_playlist_manager_n_actions = G_N_ELEMENTS (rb_playlist_manager_actions);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rb_playlist_manager_get_type (void)
-{
-	static GType rb_playlist_manager_type = 0;
-
-	if (rb_playlist_manager_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBPlaylistManagerClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_playlist_manager_class_init,
-			NULL,
-			NULL,
-			sizeof (RBPlaylistManager),
-			0,
-			(GInstanceInitFunc) rb_playlist_manager_init
-		};
-
-		rb_playlist_manager_type = g_type_register_static (G_TYPE_OBJECT,
-								   "RBPlaylistManager",
-								   &our_info, 0);
-	}
-
-	return rb_playlist_manager_type;
-}
+G_DEFINE_TYPE (RBPlaylistManager, rb_playlist_manager, G_TYPE_OBJECT)
 
 static void
 rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_playlist_manager_finalize;
 
 	object_class->set_property = rb_playlist_manager_set_property;
@@ -285,12 +263,14 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0, G_TYPE_NONE);
+
+	g_type_class_add_private (klass, sizeof (RBPlaylistManagerPrivate));
 }
 
 static void
 rb_playlist_manager_init (RBPlaylistManager *mgr)
 {
-	mgr->priv = g_new0 (RBPlaylistManagerPrivate, 1);
+	mgr->priv = RB_PLAYLIST_MANAGER_GET_PRIVATE (mgr);
 
 	mgr->priv->status_queue = g_async_queue_new ();
 
@@ -311,7 +291,7 @@ reap_dead_playlist_threads (RBPlaylistManager *mgr)
 	while ((obj = g_async_queue_try_pop (mgr->priv->status_queue)) != NULL) {
 		GDK_THREADS_ENTER ();
 		g_object_unref (obj);
-		mgr->priv->outstanding_threads--;
+		g_atomic_int_add (&mgr->priv->outstanding_threads, -1);
 		GDK_THREADS_LEAVE ();
 	}
 
@@ -322,6 +302,26 @@ reap_dead_playlist_threads (RBPlaylistManager *mgr)
 	return FALSE;
 }
 
+void
+rb_playlist_manager_shutdown (RBPlaylistManager *mgr)
+{
+	g_return_if_fail (RB_IS_PLAYLIST_MANAGER (mgr));
+
+	if (mgr->priv->exiting) {
+		return;
+	}
+		
+	mgr->priv->exiting = TRUE;
+
+	g_source_remove (mgr->priv->thread_reaper_id);
+
+	rb_debug ("%d outstanding threads", g_atomic_int_get (&mgr->priv->outstanding_threads));
+	while (g_atomic_int_get (&mgr->priv->outstanding_threads) > 0) {
+		GObject *obj = g_async_queue_pop (mgr->priv->status_queue);
+		g_object_unref (obj);
+		g_atomic_int_add (&mgr->priv->outstanding_threads, -1);
+	}
+}
 
 static void
 rb_playlist_manager_finalize (GObject *object)
@@ -331,26 +331,20 @@ rb_playlist_manager_finalize (GObject *object)
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RB_IS_PLAYLIST_MANAGER (object));
 
+	rb_debug ("Finalizing playlist manager");
+
 	mgr = RB_PLAYLIST_MANAGER (object);
 
 	g_return_if_fail (mgr->priv != NULL);
 
 	g_source_remove (mgr->priv->thread_reaper_id);
 	
-	while (mgr->priv->outstanding_threads > 0) {
-		GObject *obj = g_async_queue_pop (mgr->priv->status_queue);
-		g_object_unref (obj);
-		mgr->priv->outstanding_threads--;
-	}
-	
 	g_async_queue_unref (mgr->priv->status_queue);
 
 	g_mutex_free (mgr->priv->saving_mutex);
 	g_cond_free (mgr->priv->saving_condition);
 
-	g_free (mgr->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_playlist_manager_parent_class)->finalize (object);
 }
 
 static void
@@ -397,8 +391,8 @@ rb_playlist_manager_set_property (GObject *object,
 	case PROP_SOURCE:
 	{
 		gboolean playlist_active;
-		gboolean playlist_automatic = FALSE;
 		gboolean playlist_local = FALSE;
+		gboolean can_rename;
 		GtkAction *action;
 
 		if (mgr->priv->selected_source != NULL)
@@ -408,40 +402,46 @@ rb_playlist_manager_set_property (GObject *object,
 
 		mgr->priv->selected_source = g_value_get_object (value);
 		playlist_active = RB_IS_PLAYLIST_SOURCE (mgr->priv->selected_source);
-		if (playlist_active) {
-			g_object_get (G_OBJECT (mgr->priv->selected_source), "automatic", &playlist_automatic, NULL);
+		if (playlist_active)
 			g_object_get (G_OBJECT (mgr->priv->selected_source), "is-local", &playlist_local, NULL);
-		}
 
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
 						      "MusicPlaylistSavePlaylist");
 		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
 						      "MusicPlaylistDeletePlaylist");
-		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      playlist_local && !RB_IS_PLAY_QUEUE_SOURCE (mgr->priv->selected_source), 
+			      NULL);
+
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
  						      "EditAutomaticPlaylist");
-		g_object_set (G_OBJECT (action), "sensitive", playlist_automatic, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      RB_IS_AUTO_PLAYLIST_SOURCE (mgr->priv->selected_source), NULL);
+		can_rename = rb_source_can_rename (mgr->priv->selected_source);
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
  						      "MusicPlaylistRenamePlaylist");
-		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
-		{
-			gboolean recorder_active;
-			int num_tracks = rb_entry_view_get_num_entries (rb_source_get_entry_view (mgr->priv->selected_source));
+		g_object_set (G_OBJECT (action), "sensitive", playlist_local && can_rename, NULL);
 
-			recorder_active = playlist_active && rb_recorder_enabled ()
-					&& (num_tracks > 0);
-			action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
-			g_object_set (G_OBJECT (action), "sensitive", recorder_active, NULL);
-			
-			if (playlist_active && rb_recorder_enabled ()) {
-				/* monitor for changes, to enable/disable the burn menu item */
-				g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (mgr->priv->selected_source)),
-							 "entry-added", G_CALLBACK (rb_playlist_manager_playlist_entries_changed), mgr, 0);
-				g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (mgr->priv->selected_source)),
-							 "entry-deleted", G_CALLBACK (rb_playlist_manager_playlist_entries_changed), mgr, 0);
-			}
+		/* FIXME should base this on the query model so the entry-added and entry-deleted
+		 * signals can be removed from RBEntryView (where they don't belong).
+		 */
+		if (playlist_active && rb_recorder_enabled ()) {
+			/* monitor for changes, to enable/disable the burn menu item */
+			RBEntryView *songs = rb_source_get_entry_view (mgr->priv->selected_source);
+			g_signal_connect_object (G_OBJECT (songs), "entry-added", 
+						 G_CALLBACK (rb_playlist_manager_playlist_entries_changed), 
+						 mgr, 0);
+			g_signal_connect_object (G_OBJECT (songs), "entry-deleted", 
+						 G_CALLBACK (rb_playlist_manager_playlist_entries_changed), 
+						 mgr, 0);
+			rb_playlist_manager_playlist_entries_changed (songs, NULL, mgr);
+		} else {
+			action = gtk_action_group_get_action (mgr->priv->actiongroup,
+							      "MusicPlaylistBurnPlaylist");
+			g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		}
+
 		break;
 	}
 	case PROP_SHELL:
@@ -564,9 +564,9 @@ handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri_maybe,
 	if (entry_type == RHYTHMDB_ENTRY_TYPE_SONG) {
 		if (!mgr->priv->loading_playlist) {
 			mgr->priv->loading_playlist =
-				RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
+				RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
 		}
-		rb_playlist_source_add_location (mgr->priv->loading_playlist, uri);
+		rb_static_playlist_source_add_location (mgr->priv->loading_playlist, uri, -1);
 	}
 
 	g_free (uri);
@@ -579,7 +579,7 @@ playlist_load_start_cb (TotemPlParser *parser, const char *title, RBPlaylistMana
 
 	if (!mgr->priv->loading_playlist) {
 		mgr->priv->loading_playlist =
-			RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
+			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
 	}
 }
 
@@ -642,13 +642,33 @@ append_new_playlist_source (RBPlaylistManager *mgr, RBPlaylistSource *source)
 void
 rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 {
-	char *file = g_build_filename (rb_dot_dir (), "playlists.xml", NULL);
+	char *file;
 	xmlDocPtr doc;
 	xmlNodePtr root;
 	xmlNodePtr child;
+	gboolean exists;
 	
-	if (!g_file_test (file, G_FILE_TEST_EXISTS))
+	exists = FALSE;
+	file = g_build_filename (rb_dot_dir (), "playlists.xml", NULL);
+
+	/* block saves until the playlists have loaded */
+	g_mutex_lock (mgr->priv->saving_mutex);
+
+	exists = g_file_test (file, G_FILE_TEST_EXISTS);
+	if (! exists) {
+		rb_debug ("personal playlists not found, loading defaults");
+
+		/* try global playlists */
+		g_free (file);
+		file = g_strdup (rb_file ("playlists.xml"));
+		exists = g_file_test (file, G_FILE_TEST_EXISTS);
+	}
+
+	if (! exists) {
+		rb_debug ("default playlists file not found");
+
 		goto out;
+	}
 
 	doc = xmlParseFile (file);
 	
@@ -665,11 +685,13 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 
 		playlist = rb_playlist_source_new_from_xml (mgr->priv->shell,
 							    child);
-		append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
+		if (playlist)
+			append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	}
 
 	xmlFreeDoc (doc);
 out:
+	g_mutex_unlock (mgr->priv->saving_mutex);
 	g_free (file);
 }
 
@@ -705,7 +727,7 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 	data->mgr->priv->saving = FALSE;
 	data->mgr->priv->dirty = FALSE;
 
-	g_cond_signal (data->mgr->priv->saving_condition);
+	g_cond_broadcast (data->mgr->priv->saving_condition);
 	g_mutex_unlock (data->mgr->priv->saving_mutex);
 
 	g_async_queue_push (data->mgr->priv->status_queue, data->mgr);
@@ -716,7 +738,7 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 }
 
 void
-rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
+rb_playlist_manager_save_playlists_async (RBPlaylistManager *mgr, gboolean force)
 {
 	xmlNodePtr root;
 	struct RBPlaylistManagerSaveThreadData *data;
@@ -747,11 +769,14 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 					continue;
 
 				g_object_get (G_OBJECT (source), 
-					      "dirty", &dirty, 
-					      "is-local", &local, 
+					      "is-local", &local,
 					      NULL);
-				if (local && dirty) {
-					break;
+				if (local) {
+					g_object_get (G_OBJECT (source),
+						      "dirty", &dirty,
+						      NULL);
+					if (dirty)
+						break;
 				}
 			} while (gtk_tree_model_iter_next (model, &iter));
 		}
@@ -799,21 +824,33 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
 				continue;
 
-			g_object_get (G_OBJECT (source), "is-local", &local, 
-				      NULL);
-			if (local) {
-
+			g_object_get (G_OBJECT (source), "is-local", &local, NULL);
+			if (local)
 				rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source), root);
-			}
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 
 	g_object_ref (G_OBJECT (mgr));
-	mgr->priv->outstanding_threads++;
-	
+	g_atomic_int_inc (&mgr->priv->outstanding_threads);
+
 	g_thread_create ((GThreadFunc) rb_playlist_manager_save_thread_main, data, FALSE, NULL);
 }
 
+void
+rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
+{
+	rb_debug("saving the playlists and blocking");
+	
+	rb_playlist_manager_save_playlists_async (mgr, force);
+	
+	g_mutex_lock (mgr->priv->saving_mutex);
+
+	while (mgr->priv->saving)
+		g_cond_wait (mgr->priv->saving_condition, mgr->priv->saving_mutex);
+
+	g_mutex_unlock (mgr->priv->saving_mutex);
+}
+
 static void
 rb_playlist_manager_set_dirty (RBPlaylistManager *mgr)
 {
@@ -827,14 +864,17 @@ rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 				  const char *suggested_name, 
 				  gboolean automatic)
 {
-	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->shell, 
-								automatic, 
-								TRUE, 
-								RHYTHMDB_ENTRY_TYPE_SONG));
+	RBSource *playlist;
+	if (automatic)
+		playlist = rb_auto_playlist_source_new (mgr->priv->shell, 
+							suggested_name, 
+							TRUE);
+	else
+		playlist = rb_static_playlist_source_new (mgr->priv->shell, 
+							  suggested_name, 
+							  TRUE, 
+							  RHYTHMDB_ENTRY_TYPE_SONG);
 	
-	g_object_set (G_OBJECT (playlist),
-		      "name", suggested_name ? suggested_name : "",
-		      NULL);
 	append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	rb_sourcelist_edit_source_name (mgr->priv->sourcelist, playlist);
 	rb_playlist_manager_set_dirty (mgr);
@@ -845,6 +885,121 @@ rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 	return playlist;
 }
 
+static char *
+create_name_from_selection_data (RBPlaylistManager *mgr,
+				 GtkSelectionData *data)
+{
+	char  *name = NULL;
+	GList *list;
+
+        if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
+                list = gnome_vfs_uri_list_parse ((char *) data->data);
+
+		if (list != NULL) {
+			GList   *l;
+			char    *artist;
+			char    *album;
+			gboolean mixed_artists;
+			gboolean mixed_albums;
+
+			artist = NULL;
+			album  = NULL;
+			mixed_artists = FALSE;
+			mixed_albums  = FALSE;
+			for (l = list; l != NULL; l = g_list_next (l)) {
+				RhythmDBEntry *entry;
+				char          *location;
+				const char    *e_artist;
+				const char    *e_album;
+
+				location = gnome_vfs_uri_to_string ((const GnomeVFSURI *) l->data, 0);
+				if (location == NULL) {
+					continue;
+				}
+
+				entry = rhythmdb_entry_lookup_by_location (mgr->priv->db, location);
+				if (entry == NULL) {
+					continue;
+				}
+
+				e_artist = rb_refstring_get (entry->artist);
+				e_album  = rb_refstring_get (entry->album);
+
+				/* get value of first non-NULL artist */
+				if (e_artist != NULL && artist == NULL) {
+					artist = g_strdup (e_artist);
+				}
+
+				/* get value of first non-NULL album */
+				if (e_album != NULL && album == NULL) {
+					album = g_strdup (e_album);
+				}
+
+				/* pretend that NULL fields always match */
+				if (artist != NULL && e_artist != NULL
+				    && strcmp (artist, e_artist) != 0) {
+					mixed_artists = TRUE;
+				}
+
+				/* pretend that NULL fields always match */
+				if (album != NULL && e_album != NULL
+				    && strcmp (album, e_album) != 0) {
+					mixed_albums = TRUE;
+				}
+
+				/* if there is a mix of both then stop */
+				if (mixed_artists && mixed_albums) {
+					break;
+				}
+			}
+
+			if (! mixed_artists && ! mixed_albums) {
+				name = g_strdup_printf ("%s - %s", artist, album);
+			} else if (! mixed_artists) {
+				name = g_strdup_printf ("%s", artist);
+			} else if (! mixed_albums) {
+				name = g_strdup_printf ("%s", album);
+			}
+
+			g_free (artist);
+			g_free (album);
+			gnome_vfs_uri_list_free (list);
+		}
+
+	} else {
+		char **names;
+
+		names = g_strsplit ((char *)data->data, "\r\n", 0);
+		name = g_strjoinv (", ", names);
+		g_strfreev (names);
+	}
+
+	if (name == NULL) {
+		name = g_strdup (_("Untitled Playlist"));
+	}
+
+	return name;
+}
+
+RBSource *
+rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
+						      GtkSelectionData *data)
+{
+	RBSource *playlist;
+	gboolean  automatic;
+	char     *suggested_name;
+
+	automatic = (data->type != gdk_atom_intern ("text/uri-list", TRUE));
+	suggested_name = create_name_from_selection_data (mgr, data);
+
+	playlist = rb_playlist_manager_new_playlist (mgr,
+						     suggested_name, 
+						     automatic);
+	g_free (suggested_name);
+
+	return playlist;
+}
+
 GList *
 rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 {
@@ -870,6 +1025,8 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 			source = g_value_get_pointer (&v);
 			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
 				continue;
+			if (RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE)
+				continue;
 			g_object_get (G_OBJECT (source), "is-local", &local, 
 				      NULL);
 			if (local) {
@@ -891,7 +1048,7 @@ rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 
 static void
 rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
-					    RBPlaylistSource *playlist,
+					    RBAutoPlaylistSource *playlist,
 					    RBQueryCreator *creator)
 {
 	RBQueryCreatorLimitType type;
@@ -911,10 +1068,10 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 
 	rb_query_creator_get_sort_order (creator, &sort_key, &sort_direction);
 
-	rb_playlist_source_set_query (playlist,
-				      rb_query_creator_get_query (creator),
-				      limit_count, limit_size, limit_time,
-				      sort_key, sort_direction);
+	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (playlist),
+					   rb_query_creator_get_query (creator),
+					   limit_count, limit_size, limit_time,
+					   sort_key, sort_direction);
 }
 
 static void
@@ -924,8 +1081,7 @@ rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 	RBQueryCreator *creator = RB_QUERY_CREATOR (rb_query_creator_new (mgr->priv->db));
 	RBSource *playlist;
 	
-	switch (gtk_dialog_run (GTK_DIALOG (creator)))
-	{
+	switch (gtk_dialog_run (GTK_DIALOG (creator))) {
 	case GTK_RESPONSE_NONE:
 	case GTK_RESPONSE_CLOSE:
 		gtk_widget_destroy (GTK_WIDGET (creator));	
@@ -934,7 +1090,7 @@ rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 
 	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
 
-	rb_playlist_manager_set_automatic_playlist (mgr, RB_PLAYLIST_SOURCE (playlist), creator); 
+	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator); 
 
 	rb_playlist_manager_set_dirty (mgr);
 
@@ -946,15 +1102,15 @@ rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
 						 RBPlaylistManager *mgr)
 {
 	RBQueryCreator *creator;
-	RBPlaylistSource *playlist;
+	RBAutoPlaylistSource *playlist;
 	GPtrArray *query;
 	guint limit_count = 0, limit_size = 0, limit_time = 0;
 	const char *sort_key;
 	gint sort_direction;
 	
 
-	playlist = RB_PLAYLIST_SOURCE (mgr->priv->selected_source);
-	rb_playlist_source_get_query (playlist, &query, &limit_count, &limit_size, &limit_time, &sort_key, &sort_direction);
+	playlist = RB_AUTO_PLAYLIST_SOURCE (mgr->priv->selected_source);
+	rb_auto_playlist_source_get_query (playlist, &query, &limit_count, &limit_size, &limit_time, &sort_key, &sort_direction);
 
 	creator = RB_QUERY_CREATOR (rb_query_creator_new_from_query (mgr->priv->db,
 								     query,
@@ -1026,7 +1182,7 @@ rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 	dialog = rb_file_chooser_new (_("Load Playlist"),
 				      GTK_WINDOW (mgr->priv->window),
 				      GTK_FILE_CHOOSER_ACTION_OPEN,
-				      TRUE);
+				      FALSE);
 
 	g_signal_connect_object (G_OBJECT (dialog), "response",
 				 G_CALLBACK (load_playlist_response_cb), mgr, 0);
@@ -1064,7 +1220,7 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	dialog = rb_file_chooser_new (_("Save Playlist"),
 				      GTK_WINDOW (mgr->priv->window),
 				      GTK_FILE_CHOOSER_ACTION_SAVE,
-				      TRUE);
+				      FALSE);
 
 	g_signal_connect_object (G_OBJECT (dialog), "response",
 				 G_CALLBACK (save_playlist_response_cb),
@@ -1081,8 +1237,15 @@ rb_playlist_manager_cmd_burn_playlist (GtkAction *action,
 static void
 rb_playlist_manager_playlist_entries_changed (RBEntryView *entry_view, RhythmDBEntry *entry, RBPlaylistManager *mgr)
 {
-	int num_tracks = rb_entry_view_get_num_entries (entry_view);
-	GtkAction *action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
+	RhythmDBQueryModel *model;
+	int num_tracks;
+	GtkAction *action;
 
+	g_object_get (G_OBJECT (mgr->priv->selected_source), "query-model", &model, NULL);
+	num_tracks = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
+
+	action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
 	g_object_set (G_OBJECT (action), "sensitive", (num_tracks > 0), NULL);
+
+	g_object_unref (G_OBJECT (model));
 }
