@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of preferences dialog object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
@@ -21,22 +22,13 @@
  */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkcheckbutton.h>
+
+#include <string.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <libgnome/gnome-help.h>
-#include <string.h>
 
 #include "rb-file-helpers.h"
 #include "rb-shell-preferences.h"
@@ -47,10 +39,6 @@
 #include "eel-gconf-extensions.h"
 #include "rb-preferences.h"
 
-#ifdef WITH_AUDIOSCROBBLER
-#include "rb-audioscrobbler.h"
-#endif
-
 static void rb_shell_preferences_class_init (RBShellPreferencesClass *klass);
 static void rb_shell_preferences_init (RBShellPreferences *shell_preferences);
 static void rb_shell_preferences_finalize (GObject *object);
@@ -97,6 +85,8 @@ struct RBShellPreferencesPrivate
 	gboolean loading;
 };
 
+#define RB_SHELL_PREFERENCES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SHELL_PREFERENCES, RBShellPreferencesPrivate))
+
 G_DEFINE_TYPE (RBShellPreferences, rb_shell_preferences, GTK_TYPE_DIALOG)
 
 static void
@@ -105,6 +95,8 @@ rb_shell_preferences_class_init (RBShellPreferencesClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = rb_shell_preferences_finalize;
+
+	g_type_class_add_private (klass, sizeof (RBShellPreferencesPrivate));
 }
 
 static void
@@ -128,12 +120,9 @@ static void
 rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 {
 	GtkWidget *help;
-	GtkWidget *visible_columns_label;
 	GladeXML *xml;
-	PangoAttrList *pattrlist;
-	PangoAttribute *attr;
 
-	shell_preferences->priv = g_new0 (RBShellPreferencesPrivate, 1);
+	shell_preferences->priv = RB_SHELL_PREFERENCES_GET_PRIVATE (shell_preferences);
 
 	g_signal_connect_object (G_OBJECT (shell_preferences),
 				 "delete_event",
@@ -172,14 +161,7 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				"general_vbox",
 				shell_preferences);
 
-	pattrlist = pango_attr_list_new ();
-	attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-	attr->start_index = 0;
-	attr->end_index = G_MAXINT;
-	pango_attr_list_insert (pattrlist, attr);
-	visible_columns_label = glade_xml_get_widget (xml, "visible_columns_label");
-	gtk_label_set_attributes (GTK_LABEL (visible_columns_label), pattrlist);
-	pango_attr_list_unref (pattrlist);
+	rb_glade_boldify_label (xml, "visible_columns_label");
 	
 	/* Columns */
 	shell_preferences->priv->artist_check =
@@ -230,33 +212,37 @@ rb_shell_preferences_finalize (GObject *object)
 
 	g_return_if_fail (shell_preferences->priv != NULL);
 
-	g_free (shell_preferences->priv);
-
 	G_OBJECT_CLASS (rb_shell_preferences_parent_class)->finalize (object);
 }
 
+void
+rb_shell_preferences_append_page (RBShellPreferences *prefs,
+				  const char *name,
+				  GtkWidget *widget)
+{
+	GtkWidget *label;
+		
+	label = gtk_label_new (name);
+	gtk_notebook_append_page (GTK_NOTEBOOK (prefs->priv->notebook),
+				  widget,
+				  label);
+}
+
 static void
 rb_shell_preferences_append_view_page (RBShellPreferences *prefs,
 				       const char *name,
 				       RBSource *source)
 {
 	GtkWidget *widget;
-	GtkWidget *label;
-	gboolean visible;
 
 	g_return_if_fail (RB_IS_SHELL_PREFERENCES (prefs));
 	g_return_if_fail (RB_IS_SOURCE (source));
 
-	widget = rb_source_get_config_widget (source);
+	widget = rb_source_get_config_widget (source, prefs);
 	if (!widget)
 		return;
-	label = gtk_label_new (name);
-	g_object_get (source, "visibility", &visible, NULL);
-	if (!visible)
-		gtk_widget_set_sensitive (label, FALSE);
-	gtk_notebook_append_page (GTK_NOTEBOOK (prefs->priv->notebook),
-				  widget,
-				  label);
+
+	rb_shell_preferences_append_page (prefs, name, widget);
 }
 
 #ifdef WITH_DAAP_SUPPORT
@@ -330,50 +316,6 @@ add_daap_preferences (RBShellPreferences *shell_preferences)
 }
 #endif /* WITH_DAAP_SUPPORT */
 
-#ifdef WITH_AUDIOSCROBBLER
-static void
-rb_shell_preferences_append_audioscrobbler_config (RBShellPreferences *prefs,
-						   RBAudioscrobbler *audioscrobbler)
-{
-	GtkWidget *widget;
-
-	g_return_if_fail (RB_IS_SHELL_PREFERENCES (prefs));
-
-	widget = rb_audioscrobbler_get_config_widget (audioscrobbler);
-	if (widget) {
-		rb_debug ("got widget!");
-		gtk_notebook_append_page (GTK_NOTEBOOK (prefs->priv->notebook),
-					  widget,
-					  gtk_label_new (_("Audioscrobbler")));
-	} else {
-		rb_debug ("No config widget for Audioscrobbler");
-	}
-}
-
-GtkWidget *
-rb_shell_preferences_new (GList *views,
-			  RBAudioscrobbler *audioscrobbler)
-{
-	RBShellPreferences *shell_preferences;
-
-	shell_preferences = g_object_new (RB_TYPE_SHELL_PREFERENCES,
-				          NULL);
-
-	g_return_val_if_fail (shell_preferences->priv != NULL, NULL);
-
-	for (; views; views = views->next)
-	{
-		const char *name = NULL;
-		g_object_get (G_OBJECT (views->data), "name", &name, NULL);
-		g_assert (name != NULL);
-		rb_shell_preferences_append_view_page (shell_preferences,
-						       name,
-						       RB_SOURCE (views->data));
-	}
-
-	rb_shell_preferences_append_audioscrobbler_config (shell_preferences,
-							   audioscrobbler);
-#else
 GtkWidget *
 rb_shell_preferences_new (GList *views)
 {
@@ -393,7 +335,6 @@ rb_shell_preferences_new (GList *views)
 						       name,
 						       RB_SOURCE (views->data));
 	}
-#endif /* WITH_AUDIOSCROBBLER */
 
 #ifdef WITH_DAAP_SUPPORT
 	add_daap_preferences (shell_preferences);
