@@ -21,19 +21,17 @@
  *
  */
 
-#include <gtk/gtk.h>
 #include <config.h>
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "rb-statusbar.h"
-#include "rb-preferences.h"
-#include "rb-search-entry.h"
 #include "rb-debug.h"
-#include "eel-gconf-extensions.h"
-#include "rb-play-order.h"
 
 #define EPSILON		(0.00001)
 
@@ -52,30 +50,11 @@ static void rb_statusbar_get_property (GObject *object,
 				       GValue *value,
 				       GParamSpec *pspec);
 static void rb_statusbar_sync_with_source (RBStatusbar *statusbar);
-static void rb_statusbar_shell_play_order_changed_cb (RBShellPlayer *player, 
-						      GParamSpec *param,
-						      RBStatusbar *statusbar);
-static void rb_statusbar_state_changed_cb (GConfClient *client,
-					   guint cnxn_id,
-					   GConfEntry *entry,
-					   RBStatusbar *statusbar);
 
 static void rb_statusbar_sync_status (RBStatusbar *status);
 static gboolean poll_status (RBStatusbar *status);
-static void rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
-					    RBStatusbar *statusbar);
-static void rb_statusbar_view_statusbar_changed_cb (GtkAction *action,
-						    RBStatusbar *statusbar);
-static void rb_statusbar_entry_view_changed_cb (RBEntryView *view,
-						RBStatusbar *statusbar);
-
-static GtkToggleActionEntry rb_statusbar_toggle_entries [] =
-{
-        { "ViewStatusbar", NULL, N_("S_tatusbar"), NULL,
-	  N_("Change the visibility of the statusbar"),
-	  G_CALLBACK (rb_statusbar_view_statusbar_changed_cb), TRUE }
-};
-static guint rb_statusbar_n_toggle_entries = G_N_ELEMENTS (rb_statusbar_toggle_entries);
+static void rb_statusbar_source_status_changed_cb (RBSource *source,
+						   RBStatusbar *statusbar);
 
 struct RBStatusbarPrivate
 {
@@ -83,17 +62,9 @@ struct RBStatusbarPrivate
 
         RhythmDB *db;
 
-	gboolean syncing_state;
-
         GtkActionGroup *actiongroup;
-        RBShellPlayer *player;
-
         GtkTooltips *tooltips;
 
-        GtkWidget *shuffle;
-        GtkWidget *repeat;
-        GtkWidget *status;
-
         GtkWidget *progress;
         double progress_fraction;
         gboolean progress_changed;
@@ -105,21 +76,18 @@ struct RBStatusbarPrivate
 
         gboolean idle;
         guint idle_tick_id;
-
-	guint notify_id;
 };
 
+#define RB_STATUSBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_STATUSBAR, RBStatusbarPrivate))
+
 enum
 {
         PROP_0,
         PROP_DB,
         PROP_ACTION_GROUP,
-        PROP_SOURCE,
-        PROP_PLAYER
+        PROP_SOURCE
 };
 
-static GObjectClass *parent_class = NULL;
-
 G_DEFINE_TYPE (RBStatusbar, rb_statusbar, GTK_TYPE_STATUSBAR)
 
 static void
@@ -127,8 +95,6 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->constructor = rb_statusbar_construct;
         object_class->finalize = rb_statusbar_finalize;
 
@@ -157,15 +123,7 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               GTK_TYPE_ACTION_GROUP,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-        g_object_class_install_property (object_class,
-                                         PROP_PLAYER,
-                                         g_param_spec_object ("player",
-                                                              "RbShellPlayer",
-                                                              "RbShellPlayer object",
-                                                              RB_TYPE_SHELL_PLAYER,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-
+	g_type_class_add_private (klass, sizeof (RBStatusbarPrivate));
 }
 
 static GObject*
@@ -174,25 +132,17 @@ rb_statusbar_construct (GType                  type,
                         GObjectConstructParam *construct_properties)
 {
         RBStatusbarClass *klass;
-        GObjectClass *parent_class;  
         GObject *object;
         RBStatusbar *statusbar;
 
         klass = RB_STATUSBAR_CLASS (g_type_class_peek (RB_TYPE_STATUSBAR));
-        parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-        object = parent_class->constructor (type,
-                                            n_construct_properties,
-                                            construct_properties);
+        object = G_OBJECT_CLASS (rb_statusbar_parent_class)->constructor
+					(type,
+					 n_construct_properties,
+					 construct_properties);
         
         statusbar = RB_STATUSBAR (object);
 
-        rb_statusbar_sync_state (statusbar);
-
-        g_signal_connect_object (G_OBJECT (statusbar->priv->player),
-				 "notify::play-order", 
-				 G_CALLBACK (rb_statusbar_shell_play_order_changed_cb), 
-				 statusbar, 0);
-
         return object;
 }
 
@@ -200,27 +150,13 @@ rb_statusbar_construct (GType                  type,
 static void
 rb_statusbar_init (RBStatusbar *statusbar)
 {
-        statusbar->priv = g_new0 (RBStatusbarPrivate, 1);
+	statusbar->priv = RB_STATUSBAR_GET_PRIVATE (statusbar);
 
         statusbar->priv->tooltips = gtk_tooltips_new ();
         gtk_tooltips_enable (statusbar->priv->tooltips);
 
         gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), TRUE);
 
-        statusbar->priv->shuffle = gtk_check_button_new_with_mnemonic (_("Sh_uffle"));
-        gtk_tooltips_set_tip (GTK_TOOLTIPS (statusbar->priv->tooltips), 
-                              GTK_WIDGET (statusbar->priv->shuffle), 
-                              _("Play songs in a random order"), NULL);
-
-        statusbar->priv->repeat = gtk_check_button_new_with_mnemonic (_("_Repeat"));
-        gtk_tooltips_set_tip (GTK_TOOLTIPS (statusbar->priv->tooltips), 
-                              GTK_WIDGET (statusbar->priv->repeat), 
-                              _("Play first song again after all songs are played"), NULL);
-        g_signal_connect_object (G_OBJECT (statusbar->priv->shuffle), "toggled",
-                                 G_CALLBACK (rb_statusbar_toggle_changed_cb), statusbar, 0);
-        g_signal_connect_object (G_OBJECT (statusbar->priv->repeat), "toggled",
-                                 G_CALLBACK (rb_statusbar_toggle_changed_cb), statusbar, 0);
-
         statusbar->priv->progress = gtk_progress_bar_new ();
         statusbar->priv->progress_fraction = 1.0;
 
@@ -229,17 +165,8 @@ rb_statusbar_init (RBStatusbar *statusbar)
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->priv->progress), 1.0);
         gtk_widget_hide (statusbar->priv->progress);
 
-        gtk_box_pack_start (GTK_BOX (statusbar),
-                            GTK_WIDGET (statusbar->priv->repeat), FALSE, TRUE, 0);
-        gtk_box_pack_start (GTK_BOX (statusbar),
-                            GTK_WIDGET (statusbar->priv->shuffle), FALSE, TRUE, 0);
         gtk_box_pack_start (GTK_BOX (statusbar),
                             GTK_WIDGET (statusbar->priv->progress), FALSE, TRUE, 0);
-
-        statusbar->priv->notify_id = 
-                eel_gconf_notification_add (CONF_UI_STATUSBAR_HIDDEN,
-					    (GConfClientNotifyFunc) rb_statusbar_state_changed_cb,
-                                            statusbar);
 }
 
 static void
@@ -254,7 +181,8 @@ rb_statusbar_finalize (GObject *object)
 
         g_return_if_fail (statusbar->priv != NULL);
 
-	eel_gconf_notification_remove (statusbar->priv->notify_id);
+	g_free (statusbar->priv->loading_text);
+	g_free (statusbar->priv->progress_text);
 
         if (statusbar->priv->idle_tick_id) {
                 g_source_remove (statusbar->priv->idle_tick_id);
@@ -263,9 +191,7 @@ rb_statusbar_finalize (GObject *object)
         if (statusbar->priv->status_poll_id)
                 g_source_remove (statusbar->priv->status_poll_id);
         
-        g_free (statusbar->priv);
-
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (rb_statusbar_parent_class)->finalize (object);
 }
 
 
@@ -352,24 +278,19 @@ rb_statusbar_set_property (GObject *object,
                 break;
         case PROP_SOURCE:
                 if (statusbar->priv->selected_source != NULL) {
-                        RBEntryView *songs = rb_source_get_entry_view (statusbar->priv->selected_source);
-
-                        g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
-                                                              G_CALLBACK (rb_statusbar_entry_view_changed_cb),
-                                                              statusbar);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (statusbar->priv->selected_source),
+							      G_CALLBACK (rb_statusbar_source_status_changed_cb),
+							      statusbar);
                 }
                 
                 statusbar->priv->selected_source = g_value_get_object (value);
                 rb_debug ("selected source %p", g_value_get_object (value));
 
                 if (statusbar->priv->selected_source != NULL) {
-                        RBEntryView *songs = rb_source_get_entry_view (statusbar->priv->selected_source);
-
-                        g_signal_connect_object (G_OBJECT (songs),
-                                                 "changed",
-                                                 G_CALLBACK (rb_statusbar_entry_view_changed_cb),
-                                                 statusbar, 0);
-
+			g_signal_connect_object (G_OBJECT (statusbar->priv->selected_source),
+						 "status-changed",
+						 G_CALLBACK (rb_statusbar_source_status_changed_cb),
+						 statusbar, 0);
                 }
                 rb_statusbar_sync_with_source (statusbar);
 
@@ -387,14 +308,6 @@ rb_statusbar_set_property (GObject *object,
                                          G_CALLBACK (rb_statusbar_connect_action_group),
                                          statusbar,
                                          G_CONNECT_SWAPPED);
-
-		gtk_action_group_add_toggle_actions (statusbar->priv->actiongroup,
-						     rb_statusbar_toggle_entries,
-						     rb_statusbar_n_toggle_entries,
-						     statusbar);
-                break;
-        case PROP_PLAYER:
-                statusbar->priv->player = g_value_get_object (value);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -421,9 +334,6 @@ rb_statusbar_get_property (GObject *object,
         case PROP_ACTION_GROUP:
                 g_value_set_object (value, statusbar->priv->actiongroup);
                 break;
-        case PROP_PLAYER:
-                g_value_set_object (value, statusbar->priv->player);
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -490,14 +400,16 @@ rb_statusbar_sync_status (RBStatusbar *status)
                 pulse_progress = TRUE;
         }
 
-        /* 3. entry view busy? */
+        /* 3. query model busy? */
         if (status_text == NULL && status->priv->selected_source) {
-                RBEntryView *view;
-                view = rb_source_get_entry_view (status->priv->selected_source);
-                if (rb_entry_view_busy (view))
+		RhythmDBQueryModel *model;
+
+		g_object_get (G_OBJECT (status->priv->selected_source), "query-model", &model, NULL);
+                if (rhythmdb_query_model_has_pending_changes (model))
                         pulse_progress = TRUE;
                 else
                         show_progress = FALSE;
+		g_object_unref (G_OBJECT (model));
         }
 
         /* set up the status text */
@@ -557,12 +469,10 @@ poll_status (RBStatusbar *status)
 }
 
 RBStatusbar *
-rb_statusbar_new (RhythmDB *db, GtkActionGroup *actions, 
-                  RBShellPlayer *player)
+rb_statusbar_new (RhythmDB *db, GtkActionGroup *actions)
 {
         RBStatusbar *statusbar = g_object_new (RB_TYPE_STATUSBAR,
                                                "db", db,
-                                               "player", player,
                                                "action-group", actions,
                                                NULL);
 
@@ -602,101 +512,10 @@ rb_statusbar_sync_with_source (RBStatusbar *statusbar)
 	g_free (status_str);
 }
 
-void
-rb_statusbar_sync_state (RBStatusbar *statusbar)
-{
-	GtkAction *action;
-        gboolean hidden, consistent;
-        int shuffle, repeat;
-
-        rb_debug ("syncing state");
-
-	statusbar->priv->syncing_state = TRUE;
-        consistent = rb_shell_player_get_playback_state (statusbar->priv->player,
-                                                         &shuffle, &repeat);
-        if (consistent) {
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
-                                              shuffle);
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->repeat),
-                                              repeat);
-        } 
-        gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
-                                            !consistent);
-        gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (statusbar->priv->repeat),
-                                            !consistent);
-        
-        hidden = eel_gconf_get_boolean (CONF_UI_STATUSBAR_HIDDEN);
-        if (hidden)
-                gtk_widget_hide (GTK_WIDGET (statusbar));
-        else
-                gtk_widget_show (GTK_WIDGET (statusbar));
-	action = gtk_action_group_get_action (statusbar->priv->actiongroup,
-					      "ViewStatusbar");
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), !hidden);
-	statusbar->priv->syncing_state = FALSE;
-}
-	
 static void
-rb_statusbar_state_changed_cb (GConfClient *client,
-                               guint cnxn_id,
-                               GConfEntry *entry,
-                               RBStatusbar *statusbar)
+rb_statusbar_source_status_changed_cb (RBSource *source, RBStatusbar *statusbar)
 {
-        rb_debug ("state changed");
-	if (statusbar->priv->syncing_state)
-		return;
-	statusbar->priv->syncing_state = TRUE;
-	rb_statusbar_sync_state (statusbar);
-}
-
-static void
-rb_statusbar_shell_play_order_changed_cb (RBShellPlayer *player, 
-					  GParamSpec *param, 
-					  RBStatusbar *statusbar)
-{
-        rb_debug ("state changed");
-	if (statusbar->priv->syncing_state)
-		return;
-	statusbar->priv->syncing_state = TRUE;
-	rb_statusbar_sync_state (statusbar);
-}
-
-static void
-rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
-                                RBStatusbar *statusbar)
-{
-        RBShellPlayer *player = statusbar->priv->player;
-        gboolean shuffle, repeat;
-
-	if (statusbar->priv->syncing_state)
-		return;
-	statusbar->priv->syncing_state = TRUE;
-
-        rb_debug ("toggle changed");
-
-        shuffle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle));
-        repeat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (statusbar->priv->repeat));
-
-        rb_shell_player_set_playback_state (player, shuffle, repeat);
-	statusbar->priv->syncing_state = FALSE;
-}
-
-static void
-rb_statusbar_view_statusbar_changed_cb (GtkAction *action,
-					RBStatusbar *statusbar)
-{
-        rb_debug ("got view statusbar toggle: active: %d");
-	if (statusbar->priv->syncing_state)
-		return;
-	eel_gconf_set_boolean (CONF_UI_STATUSBAR_HIDDEN,
-			       !gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
-}
-
-static void
-rb_statusbar_entry_view_changed_cb (RBEntryView *view,
-                                    RBStatusbar *statusbar)
-{
-        rb_debug ("entry view changed");
-        if (statusbar->priv->idle)
-                rb_statusbar_sync_with_source (statusbar);
+	rb_debug ("source status changed");
+	if (statusbar->priv->idle)
+		rb_statusbar_sync_with_source (statusbar);
 }
