@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  Implementation of DAAP (iTunes Music Sharing) source object
  *
  *  Copyright (C) 2005 Charles Schmidt <cschmidt2@emich.edu>
@@ -21,13 +22,13 @@
 
 #include <config.h>
 
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkiconfactory.h>
 #include <string.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
 #include "rhythmdb.h"
 #include "rb-shell.h"
-#include <libgnome/gnome-i18n.h>
 #include "eel-gconf-extensions.h"
 #include "rb-daap-source.h"
 #include "rb-stock-icons.h"
@@ -38,9 +39,8 @@
 
 #include "rb-daap-connection.h"
 #include "rb-daap-mdns.h"
-#include "rb-daap-src.h"
 
-#include "rb-playlist-source.h"
+#include "rb-static-playlist-source.h"
 
 
 static void rb_daap_source_dispose (GObject *object);
@@ -85,6 +85,7 @@ struct RBDAAPSourcePrivate
 	GSList *playlist_sources;
 };
 
+#define RB_DAAP_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_DAAP_SOURCE, RBDAAPSourcePrivate))
 
 enum {
 	PROP_0,
@@ -164,12 +165,14 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 							       "Whether the share is password protected",
 							       FALSE,
 							       G_PARAM_READWRITE));
+
+	g_type_class_add_private (klass, sizeof (RBDAAPSourcePrivate));
 }
 
 static void
 rb_daap_source_init (RBDAAPSource *source)
 {
-	source->priv = g_new0 (RBDAAPSourcePrivate, 1);
+	source->priv = RB_DAAP_SOURCE_GET_PRIVATE (source);
 }
 
 
@@ -184,8 +187,6 @@ rb_daap_source_dispose (GObject *object)
 	
 		g_free (source->priv->service_name);
 		g_free (source->priv->host);
-		g_free (source->priv);
-		source->priv = NULL;
 	}
 
 	G_OBJECT_CLASS (rb_daap_source_parent_class)->dispose (object);
@@ -430,7 +431,6 @@ start_browsing (RBShell *shell)
 							  (GDestroyNotify)g_free, 
 							  (GDestroyNotify)stop_resolver);
 
-	rb_daap_src_init ();
 }
 
 static void
@@ -458,8 +458,6 @@ stop_browsing (RBShell *shell)
 		rb_daap_mdns_browse_cancel (browser);
 		browser = 0;
 	}
-
-	rb_daap_src_shutdown ();
 }
 
 static void
@@ -583,16 +581,8 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		RBDAAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
 
-		playlist_source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
-							   "name", playlist->name,
-							   "shell", shell,
-							   "visibility", TRUE,
-							   "is-local", FALSE,
-							   "entry-type", entry_type,
-							   NULL));
-		/* this is set here instead of in construction so that
-		 * rb_playlist_source_constructor has a chance to be run to set things up */
-		rb_playlist_source_add_locations (RB_PLAYLIST_SOURCE (playlist_source), playlist->uris);
+		playlist_source = rb_static_playlist_source_new (shell, playlist->name, FALSE, entry_type);
+		rb_static_playlist_source_add_locations (RB_STATIC_PLAYLIST_SOURCE (playlist_source), playlist->uris);
 
 		rb_shell_append_source (shell, playlist_source, RB_SOURCE (daap_source));
 		daap_source->priv->playlist_sources = g_slist_prepend (daap_source->priv->playlist_sources, playlist_source);
