@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of Internet Radio source object
  *
  *  Copyright (C) 2002,2003 Colin Walters <walters@debian.org>
@@ -20,12 +21,14 @@
  */
 
 #include <config.h>
+
+#include <string.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libxml/tree.h>
-#include <string.h>
 
 #include "rb-iradio-source.h"
 
@@ -144,6 +147,8 @@ struct RBIRadioSourcePrivate
 	RhythmDBQueryModel *all_query;
 };
 
+#define RB_IRADIO_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IRADIO_SOURCE, RBIRadioSourcePrivate))
+
 static const GtkTargetEntry stations_view_drag_types[] = {
 	{  "text/uri-list", 0, 0 },
 	{  "_NETSCAPE_URL", 0, 1 },
@@ -171,6 +176,7 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	object_class->get_property = rb_iradio_source_get_property;
 
 	source_class->impl_get_status  = impl_get_status;
+	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
@@ -195,14 +201,16 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 							    G_MAXINT,
 							    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	
+
+	g_type_class_add_private (klass, sizeof (RBIRadioSourcePrivate));
 }
 
 static void
 rb_iradio_source_init (RBIRadioSource *source)
 {
 	gint size;
-	source->priv = g_new0 (RBIRadioSourcePrivate, 1);
+
+	source->priv = RB_IRADIO_SOURCE_GET_PRIVATE (source);
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
@@ -257,8 +265,6 @@ rb_iradio_source_finalize (GObject *object)
 	g_free (source->priv->title);
 	g_free (source->priv->status);
 
-	g_free (source->priv);
-
 	G_OBJECT_CLASS (rb_iradio_source_parent_class)->finalize (object);
 }
 
@@ -269,6 +275,7 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	RBIRadioSource *source;
 	RBIRadioSourceClass *klass;
 	RBShell *shell;
+	GObject *shell_player;
 
 	klass = RB_IRADIO_SOURCE_CLASS (g_type_class_peek (RB_TYPE_IRADIO_SOURCE));
 
@@ -279,11 +286,13 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 
 	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
 	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	shell_player = rb_shell_get_player (shell);
 	g_object_unref (G_OBJECT (shell));
 
 
 	/* set up stations view */
-	source->priv->stations = rb_entry_view_new (source->priv->db, CONF_STATE_IRADIO_SORTING,
+	source->priv->stations = rb_entry_view_new (source->priv->db, shell_player,
+						    CONF_STATE_IRADIO_SORTING,
 						    FALSE, FALSE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
@@ -361,8 +370,7 @@ rb_iradio_source_set_property (GObject *object,
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
 		source->priv->entry_type = g_value_get_uint (value);
 		break;
@@ -380,8 +388,7 @@ rb_iradio_source_get_property (GObject *object,
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
 		g_value_set_uint (value, source->priv->entry_type);
 		break;
@@ -420,6 +427,9 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 		return;
 	}
 	entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
+	if (entry == NULL)
+		return;
+
 	g_value_init (&val, G_TYPE_STRING);
 	if (title)
 		g_value_set_static_string (&val, title);
@@ -527,9 +537,14 @@ rb_iradio_source_async_update_play_statistics (gpointer data)
 static char *
 impl_get_status (RBSource *asource)
 {
- 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
 	char *ret;
-	guint num_entries = rb_entry_view_get_num_entries (source->priv->stations);
+	RhythmDBQueryModel *model;
+	guint num_entries;
+
+	g_object_get (G_OBJECT (asource), "query-model", &model, NULL);
+	num_entries = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
+	g_object_unref (G_OBJECT (model));
+
 	ret = g_strdup_printf (ngettext ("%d station", "%d stations", num_entries),
 			       num_entries);
 	return ret;
@@ -558,7 +573,7 @@ static void
 impl_song_properties (RBSource *asource)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
-	GtkWidget *dialog = rb_station_properties_dialog_new (source->priv->stations, FALSE);
+	GtkWidget *dialog = rb_station_properties_dialog_new (source->priv->stations);
 	rb_debug ("in song properties");
 	if (dialog)
 		gtk_widget_show_all (dialog);
@@ -602,8 +617,8 @@ rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 						   RBIRadioSource *source)
 {
 	rb_debug ("sort order changed");
-	rb_entry_view_set_resorting (view);
-	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
+
+	rb_entry_view_resort_model (view);
 }
 
 static void
@@ -722,12 +737,7 @@ rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 	model = GTK_TREE_MODEL (query_model);
 	
 	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (query_model));
-
-	/* we rely on getting signals from all_query to update the genre view.
-	 * we keep it marked as connected so it doesn't just eat the signals.
-	 */
-	if (query_model != source->priv->all_query)
-		rhythmdb_query_model_set_connected (source->priv->all_query, TRUE);
+	g_object_set (G_OBJECT (source), "query-model", query_model, NULL);
 
 	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
 
@@ -739,9 +749,30 @@ static void
 handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri, const char *title,
 			  const char *genre, RBIRadioSource *source)
 {
-	if (!rhythmdb_entry_lookup_by_location (source->priv->db, uri)) {
-		rb_iradio_source_add_station (source, uri, title, genre);
+	rb_iradio_source_add_station (source, uri, title, genre);
+}
+
+void
+rb_iradio_source_add_from_playlist (RBIRadioSource *source,
+				    const char     *uri)
+{
+	TotemPlParser *parser = totem_pl_parser_new ();
+
+	g_signal_connect_object (G_OBJECT (parser), "entry",
+				 G_CALLBACK (handle_playlist_entry_cb),
+				 source, 0);
+	switch (totem_pl_parser_parse (parser, uri, FALSE)) {
+	case TOTEM_PL_PARSER_RESULT_UNHANDLED:
+	case TOTEM_PL_PARSER_RESULT_IGNORED:
+		/* maybe it's the actual stream URL, then */
+		rb_iradio_source_add_station (source, uri, NULL, NULL);
+		break;
+
+	case TOTEM_PL_PARSER_RESULT_SUCCESS:
+	case TOTEM_PL_PARSER_RESULT_ERROR:
+		break;
 	}
+	g_object_unref (G_OBJECT (parser));
 }
 
 static void
@@ -750,14 +781,10 @@ rb_iradio_source_first_time_changed (GConfClient *client,
 				     GConfEntry *entry,
 				     RBIRadioSource *source)
 {
-	TotemPlParser *parser = totem_pl_parser_new ();
 	char *uri = gnome_vfs_get_uri_from_local_path (rb_file ("iradio-initial.pls"));
 
-	g_signal_connect_object (G_OBJECT (parser), "entry",
-				 G_CALLBACK (handle_playlist_entry_cb),
-				 source, 0);
-	totem_pl_parser_parse (parser, uri, FALSE);	
-	g_object_unref (G_OBJECT (parser));
+	rb_iradio_source_add_from_playlist (source, uri);
+
 	g_free (uri);
 }
 
@@ -793,18 +820,17 @@ stations_view_drag_data_received_cb (GtkWidget *widget,
 	while (i != NULL) {
 		char *uri = NULL;
 
-		uri = i->data;
-
 		/* as totem source says, "Super _NETSCAPE_URL trick" */
 		if (info == 1) {
-			i = i->next;
-			if (i != NULL) {
+			if (i != NULL)
 				g_free (i->data);
-			}
+			i = i->next;
+			if (i == NULL)
+				break;
 		}
 
-		if ((uri != NULL) && 
-		    (!rhythmdb_entry_lookup_by_location (source->priv->db, uri))) {
+		uri = i->data;
+		if (uri != NULL) {
 			rb_iradio_source_add_station (source, uri, NULL, NULL);
 		}
 
