@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  * arch-tag: Implementation of Rhythmbox first-time druid
  *
  *  Copyright (C) 2003,2004 Colin Walters <walters@debian.org>
@@ -19,9 +20,10 @@
  *
  */
 
-#include <gtk/gtk.h>
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <libgnomeui/gnome-druid.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
 #include <libgnomeui/gnome-druid-page-standard.h>
@@ -33,8 +35,6 @@
 #include "rb-glade-helpers.h"
 #include "eel-gconf-extensions.h"
 
-static void rb_druid_class_init (RBDruidClass *klass);
-static void rb_druid_init (RBDruid *druid);
 static void rb_druid_finalize (GObject *object);
 static void rb_druid_set_property (GObject *object,
 				   guint prop_id,
@@ -65,51 +65,22 @@ struct RBDruidPrivate
 	GtkWidget *path_entry;
 };
 
+G_DEFINE_TYPE (RBDruid, rb_druid, G_TYPE_OBJECT)
+#define RB_DRUID_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_DRUID, RBDruidPrivate))
+
 enum
 {
 	PROP_0,
 	PROP_DB,
 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rb_druid_get_type (void)
-{
-	static GType rb_druid_type = 0;
-
-	if (rb_druid_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBDruidClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_druid_class_init,
-			NULL,
-			NULL,
-			sizeof (RBDruid),
-			0,
-			(GInstanceInitFunc) rb_druid_init
-		};
-
-		rb_druid_type = g_type_register_static (G_TYPE_OBJECT,
-							"RBDruid",
-							&our_info, 0);
-	}
-
-	return rb_druid_type;
-}
 
 static void
 rb_druid_class_init (RBDruidClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_druid_finalize;
-
 	object_class->set_property = rb_druid_set_property;
 	object_class->get_property = rb_druid_get_property;
 
@@ -120,13 +91,16 @@ rb_druid_class_init (RBDruidClass *klass)
 							      "RhythmDB object",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE));
+
+	g_type_class_add_private (klass, sizeof (RBDruidPrivate));
 }
 
 static void
 rb_druid_init (RBDruid *druid)
 {
 	GladeXML *xml;
-	druid->priv = g_new0 (RBDruidPrivate, 1);
+
+	druid->priv = RB_DRUID_GET_PRIVATE (druid);
 
 	xml = rb_glade_xml_new ("druid.glade", "druid_toplevel", druid);
 
@@ -163,9 +137,7 @@ rb_druid_finalize (GObject *object)
 	if (druid->priv->window)
 		gtk_widget_destroy (GTK_WIDGET (druid->priv->window));
 
-	g_free (druid->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_druid_parent_class)->finalize (object);
 }
 
 static void
@@ -319,7 +291,7 @@ rb_druid_browse_clicked_cb (GtkButton *button, RBDruid *druid)
 	dialog = rb_file_chooser_new (_("Load folder into Library"),
 				      GTK_WINDOW (druid->priv->window),
 				      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-				      TRUE);
+				      FALSE);
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
 	g_signal_connect_object (G_OBJECT (dialog), "response",
